"""StructuredDocument product

  A StructuredDocument object contains text in structured text markup
  which is rendered in HTML format. Certain extensions to the basic
  structured text markup have been added.

  TBD - describe extensions here

  Structured Documents are also DTML Documents, so all DTML markup
  is converted to HTML after structured text markup is converted.
"""

__version__='$Revision: 1.2 $'[11:-2]

import sys, os, string
import Globals, Acquisition, OFS.SimpleItem, AccessControl.Role, Persistence
from OFS.DTMLDocument import DTMLDocument
from Globals import MessageDialog, HTMLFile
from urllib import quote
from StructuredText2 import html_with_references
from string import replace

default_level = 1

class StructuredDocument(DTMLDocument):

    """Object that renders structured text as HTML."""

    meta_type='Structured Document'

    __ac_permissions__=(
    ('Change Structured Documents', ('manage_edit', 'manage_upload', 'PUT')),
    )
    
    _properties=({'id':'title', 'type': 'string', 'mode':'w'},
                 {'id':'level', 'type': 'int', 'mode': 'w'},
    )

    manage_editForm=HTMLFile('methodEdit', globals())
    manage=manage_main=manage_editDocument=manage_editForm

    def __init__(self, source_string='', level=default_level, 
        mapping=None, __name__='<string>', **vars):

        self.level=level
        apply(DTMLDocument.__init__, 
            (self, source_string, mapping, __name__), vars)
        
    def manage_edit(self,data,title,level,SUBMIT='Change',dtpref_cols='50',
                    dtpref_rows='20',REQUEST=None):
                    
        """Edit a Structured Document."""
        
        self._validateProxy(REQUEST)
        if self._size_changes.has_key(SUBMIT):
            return self._er(data,title,SUBMIT,dtpref_cols,dtpref_rows,REQUEST)

        self.title=title
        self.level=level
        if type(data) is not type(''): data=data.read()
        self.munge(data)
        if REQUEST:
            message="Content changed."
            return self.manage_main(self,REQUEST,manage_tabs_message=message)
    
    def read(self,raw=None):
        raw = replace(self.read_raw(), '\r\n', '\n')
        return str(html_with_references(raw, level=self.level))
        
    def __call__(self, client=None, REQUEST={}, RESPONSE=None, **kw):
        if kw.has_key('level'):
            level = self.level
            self.level = kw['level']
            r = apply(DTMLDocument.__call__, 
                (self, client, REQUEST, RESPONSE), kw)
            self.level = level
            return r
        else:
            return apply(DTMLDocument.__call__, 
                (self, client, REQUEST, RESPONSE), kw)
        
def manage_addStructuredDocument(self, id, title='', file='', level='',
    REQUEST=None, submit=None):

    """Add a StructuredDocument object with the contents of file."""
    
    if type(file) is not type(''): file=file.read()
    ob=StructuredDocument(file, __name__=id)
    ob.title=title
    ob.level=level or default_level
    id=self._setObject(id, ob)
    if REQUEST is not None:
        try: u=self.DestinationURL()
        except: u=REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''

manage_addStructuredDocumentForm = HTMLFile('methodAdd', globals())

